/*
 * application-sandboxer: an application isolation mechanism
 * Copyright (C) 2009, Intel Corporation.
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms and conditions of the GNU Lesser General Public License,
 * version 2.1, as published by the Free Software Foundation.
 *
 * This program is distributed in the hope it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
 * more details.
 *
 * You should have received a copy of the GNU Lesser General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51 Franklin St 
 * - Fifth Floor, Boston, MA 02110-1301 USA
 *
 */


/**
 * \file plugin_impl.h
 * This file is for the implementation of plugins
 * plugins themselves should not include this file
 * plugins should instead include plugin.h
 *
 */

#ifndef __PLUGIN_IMPL_H__
#define __PLUGIN_IMPL_H__

#include <config.h>
#include <daemon/plugin.h>
#include <gmodule.h>

G_BEGIN_DECLS

extern GPtrArray *plugins; /* global list of plugins */


/**
 * representation of a plugin that ties together the
 * #SandboxerPluginDescriptor and the GModule
 *
 * \param[in] desc    a #SandboxerPluginDescriptor
 * \param[in] module  a GModule
 */
struct _SandboxerPlugin {
  SandboxerPluginDescriptor *desc;
  GModule *module;
};


/**
 * plugin_system_init: call once to initialize the plugin system
 */
void
plugin_system_init(void);

/**
 * load_plugins: call to load plugins enabled plugins from the plugin
 * directory specified in #SandboxGlobalConfig
 */
void
load_plugins(void);

gboolean
sandboxer_register_plugin(const SandboxerPluginDescriptor *desc, 
			  GModule *module);


G_END_DECLS

#endif /* __PLUGIN_IMPL_H__ */
