/*
 * application-sandboxer: an application isolation mechanism
 * Copyright (C) 2009, Intel Corporation.
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms and conditions of the GNU Lesser General Public License,
 * version 2.1, as published by the Free Software Foundation.
 *
 * This program is distributed in the hope it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
 * more details.
 *
 * You should have received a copy of the GNU Lesser General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51 Franklin St 
 * - Fifth Floor, Boston, MA 02110-1301 USA
 *
 */


#ifndef  _SANDBOXER_H_
#define  _SANDBOXER_H_

#include <config.h>
#include <daemon/base.h>
#include <daemon/config_store.h>

#include <glib.h>
#include <glib-object.h>
#include <dbus/dbus-glib.h>


#define SANDBOXER_DBUS_SANDBOX_PATH "/com/intel/sandboxer/sandbox"
#define SANDBOXER_DBUS_SANDBOX      "com.intel.sandboxer.Sandbox"


#define SANDBOXER_TYPE_SANDBOX            (sandboxer_sandbox_get_type ())
#define SANDBOXER_SANDBOX(object)         (G_TYPE_CHECK_INSTANCE_CAST ((object), SANDBOXER_TYPE_SANDBOX, SandboxerSandbox))
#define SANDBOXER_SANDBOX_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST ((klass), SANDBOXER_TYPE_SANDBOX, SandboxerSandboxClass))
#define SANDBOXER_IS_SANDBOX(object)      (G_TYPE_CHECK_INSTANCE_TYPE ((object), SANDBOXER_TYPE_SANDBOX))
#define SANDBOXER_IS_SANDBOX_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), SANDBOXER_TYPE_SANDBOX))
#define SANDBOXER_SANDBOX_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS ((obj), SANDBOXER_TYPE_SANDBOX, SandboxerSandboxClass))

#define SANDBOXER_SANDBOX_ERROR sandboxer_sandbox_error_quark ()


G_BEGIN_DECLS

typedef enum {
  SANDBOXER_SANDBOX_ERROR_FORK,
  SANDBOXER_SANDBOX_ERROR_FAILED
} SandboxerSandboxError;

typedef struct _SandboxerSandboxClass SandboxerSandboxClass;
typedef struct _SandboxerSandbox SandboxerSandbox;

/**
 * class representation of two or more processes sharing the same
 * sandbox.  Can be thought of as the class representation of stubd,
 * as each stubd has only on of these.
 */
struct _SandboxerSandboxClass {
  /** _SandboxerBaseClass parent class  */
  SandboxerBaseClass parent;

  /** the dbus connection to hold */
  DBusGConnection *connection;

  /** the dbus unique connection name that we use */
  gchar *unique_name;
};


/**
 * instance representation of two or more processes sharing the same
 * sandbox.  This can be thought of as the instance representation of
 * stubd.
 * 
 */
struct _SandboxerSandbox {
  /** parent object */
  SandboxerBase parent;

  /** the jail_dir that the sandbox is in */
  gchar *sandbox_dir;
};

GType sandboxer_sandbox_get_type(void);

//you can free returned gchar*
gchar *
sandboxer_sandbox_generate_dbus_path(const GPid pid);

SandboxerSandbox*
sandboxer_sandbox_new(const gchar *sandbox_name);

gboolean
sandboxer_sandbox_get_process(SandboxerSandbox *self, const gchar *path,
			      gchar **process, gchar **secret_cookie, gchar **unique_name,
			      GError **error);

gboolean 
sandboxer_sandbox_list_all_processes(SandboxerSandbox *self, 
				     gchar ***paths, gchar ***secret_cookies,
				     gchar ***unique_names, GError **error);


gboolean 
sandboxer_sandbox_find_process_by_pid(SandboxerSandbox *self, 
				      const gint pid, gboolean *found,
				      gchar **process, gchar **secret_cookie, gchar **unique_name,
				      GError **error);

G_END_DECLS

#endif /* _SANDBOXER_H_ */
