/*
 * application-sandboxer: an application isolation mechanism
 * Copyright (C) 2009, Intel Corporation.
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms and conditions of the GNU Lesser General Public License,
 * version 2.1, as published by the Free Software Foundation.
 *
 * This program is distributed in the hope it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
 * more details.
 *
 * You should have received a copy of the GNU Lesser General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51 Franklin St 
 * - Fifth Floor, Boston, MA 02110-1301 USA
 *
 */


#include <config.h>
#include <daemon/sandboxer-helper.h>

#include <sched.h>
#include <errno.h>
#include <dbus/dbus.h>
#include <dbus/dbus-glib-lowlevel.h>

#include <daemon/fs_fixup.h>

typedef struct _cred cred;
struct _cred {
  uid_t uid;
  gid_t gid;
};

static GQueue *credq= NULL;


gboolean
is_root(void)
{
  uid_t ruid, euid, suid;

  if (getresuid(&ruid, &euid, &suid))
    g_error("getresuid: %s", g_strerror(errno));
  
  if (0 == ruid && 0 == euid && 0 == suid) {
    return TRUE;
  }
  i_am_here();
  return FALSE;
}

gboolean
could_become_root(void)
{
  uid_t ruid, euid, suid;

  if (getresuid(&ruid, &euid, &suid))
    g_error("getresuid: %s", g_strerror(errno));
  
  if (0 == suid) {
    return TRUE;
  }
  return FALSE;
}

void 
become_uid_perm(const uid_t uid)
{
  if (setresuid(uid, uid, uid)) {
    g_error("wanted to become %d, but couldn't", uid);
  }
}


void 
push_cred(const uid_t uid, const gid_t gid)
{
  cred *element;

  if (NULL == credq) {
    credq= g_queue_new();
    g_assert(credq);
  }
  element= g_slice_new0(cred); g_assert(element);
  element->uid= getuid();
  element->gid= getgid();  
  g_queue_push_head(credq, element);    

  if (setresuid(0, 0, 0)) g_error("setresuid: %s", g_strerror(errno));
  if (setresgid(gid, gid, 0)) g_error("setresgid: %s", g_strerror(errno));
  if (setresuid(uid, uid, 0)) g_error("setresuid: %s", g_strerror(errno));
}

void 
pop_cred(void)
{
  cred *element;

  if (NULL == credq) {
    g_error("tried to pop empty credential queue");
  }
  element= (cred*)g_queue_pop_head(credq); 
  if (NULL == element) g_error("popped empty queue");
  if (setresuid(0, 0, 0)) g_error("setresuid: %s", g_strerror(errno));
  if (setresgid(element->gid, element->gid, 0)) g_error("setresgid: %s", g_strerror(errno));
  if (setresuid(element->uid, element->uid, 0)) g_error("setresuid: %s", g_strerror(errno));
  g_slice_free(cred, element); 
}

const gchar* 
get_unique_connection_name(DBusGConnection *gconnection)
{
  DBusConnection *dbus_connection;
  
  g_assert(gconnection);
  dbus_connection= dbus_g_connection_get_connection(gconnection);
  g_assert(dbus_connection);
  return dbus_bus_get_unique_name(dbus_connection);
}

