/*
 * application-sandboxer: an application isolation mechanism
 * Copyright (C) 2009, Intel Corporation.
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms and conditions of the GNU Lesser General Public License,
 * version 2.1, as published by the Free Software Foundation.
 *
 * This program is distributed in the hope it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
 * more details.
 *
 * You should have received a copy of the GNU Lesser General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51 Franklin St 
 * - Fifth Floor, Boston, MA 02110-1301 USA
 *
 */


#ifndef _SANDBOXER_HELPER_H
#define _SANDBOXER_HELPER_H

#include <config.h>
#include <dbus/dbus-glib-lowlevel.h>
#include <sys/types.h>
#include <unistd.h>
#include <glib.h>

G_BEGIN_DECLS

//FIXME: debug only
#define here() g_debug("%s:%d (%s)", __FILE__, __LINE__, __FUNCTION__);


//FIXME: debug only
#define i_am_here()                               \
{                                                 \
  uid_t ruid, euid, suid;                         \
                                                  \
  getresuid(&ruid, &euid, &suid);                 \
  g_debug("(%d, %d, %d) at %s:%d (%s)",		  \
          ruid, euid, suid,			  \
	  __FILE__, __LINE__, __FUNCTION__);	  \
}                                                 

/**
 * check if current process is running as UID 0
 *
 * \returns TRUE if it is; FALSE otherwise
 */
gboolean
is_root(void);

/**
 * check if the current process could become root
 * using setresuid.  This is really a check if suid is 0
 *
 * \returns TRUE if it can; FALSE if it can't
 */
gboolean
could_become_root(void);

/**
 * drop privileges and setuid
 * 
 * \param[in]  uid    the uid to setuid to.  Note this also sets the suid to this
 *                    value, so one cannot return to root.
 */
void 
become_uid_perm(const uid_t uid);

/**
 * gets the unique connection name of a DBusGConnection
 *
 * \param[in] gconnection  a DBusGConnection
 *
 * \returns a NULL terminated string with the connection name.  This
 * string is private to get_unique_connection_name().  It is an error
 * to free it.
 */
const gchar* 
get_unique_connection_name(DBusGConnection *gconnection);

/**
 * return to the uid and gid stored by push_cred() by retrieving from
 * the opaque queue it maintains.  It is an error to call pop_cred()
 * before push_cred() or to call pop_cred() with empty queue.
 */
void 
pop_cred(void);

/**
 * become the uid and gid specified in gid and uid, and
 * push the current uid and gid on an opaque stack of credentials that
 * can be returned to later with pop_cred()
 *
 * \param[in]  uid    the uid to switch to
 * \param[in]  gid    the gid to switch to
 */
void 
push_cred(const uid_t uid, const gid_t gid);

G_END_DECLS

#endif

