/*
 * application-sandboxer: an application isolation mechanism
 * Copyright (C) 2009, Intel Corporation.
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms and conditions of the GNU Lesser General Public License,
 * version 2.1, as published by the Free Software Foundation.
 *
 * This program is distributed in the hope it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
 * more details.
 *
 * You should have received a copy of the GNU Lesser General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51 Franklin St 
 * - Fifth Floor, Boston, MA 02110-1301 USA
 *
 */


#include <config.h>
#include <daemon/sandboxer-helper.h>
#include <daemon/config_store.h>
#include <daemon/sandbox.h>
#include <daemon/plugin.h>
#include <daemon/plugin_impl.h>
#include <sys/prctl.h>
#include <stdlib.h>
#include <glib.h>
#include <errno.h>

#define CONFIG_FILE_KEY "STUBD_CONFIG"
#define DEFAULT_CONFIG_FILE "/etc/sandboxer/sandboxerd.conf"


int main (int argc, char **argv)
{
  GMainLoop *loop;
  gchar *config_file;
  SandboxConfigStore *config_store;
  SandboxGlobalConfig *global_config;
  SandboxerSandbox *sandbox;
  const gchar *sandbox_name;


  g_type_init();        
  if (!g_thread_supported())
    g_thread_init (NULL);
  dbus_g_thread_init();

  g_log_set_always_fatal(G_LOG_LEVEL_ERROR    |
			 G_LOG_LEVEL_CRITICAL |
			 G_LOG_LEVEL_WARNING);
  
  g_debug("stubd started");
  if (!is_root()) {
    g_error("stubd refusing to run as %d.  Please run as root", geteuid());
    exit(EXIT_FAILURE);
  }
  
  config_file= (gchar*)g_getenv(CONFIG_FILE_KEY);
  if (!config_file) {
    config_file= DEFAULT_CONFIG_FILE;
    g_debug("%s not defined, so using \"%s\" as config file", CONFIG_FILE_KEY, config_file);
  }
  global_config= sandbox_global_config_new(config_file);
  if (!global_config) {
    g_error("error reading config file");
    exit(EXIT_FAILURE);
  }
  i_am_here();
  push_cred(global_config->sandbox_user_uid, global_config->sandbox_user_gid);
  config_store= sandbox_config_store_new(global_config->package_file_directory);
  g_assert(config_store);

  plugin_system_init();
  load_plugins();
  
  sandbox_name= g_getenv("STUBD_SANDBOX_NAME");
  if (NULL == sandbox_name) {
    g_error("stubd refusing to run without a valid SandboxName");
  }

  sandbox= sandboxer_sandbox_new(sandbox_name); g_assert(sandbox);

  loop = g_main_loop_new (NULL, FALSE); g_assert(loop);  
  g_main_loop_run (loop);
  
  return 0; /* never happens */
}


    
