/*
 * application-sandboxer: an application isolation mechanism
 * Copyright (C) 2009, Intel Corporation.
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms and conditions of the GNU Lesser General Public License,
 * version 2.1, as published by the Free Software Foundation.
 *
 * This program is distributed in the hope it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
 * more details.
 *
 * You should have received a copy of the GNU Lesser General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51 Franklin St 
 * - Fifth Floor, Boston, MA 02110-1301 USA
 *
 */


#include <config.h>
#include <daemon/plugin.h>
#include <daemon/process.h>
#include <daemon/sandboxer-helper.h>
#include <unistd.h>

extern gchar* tempdir;

static void 
write_resuid(const gchar *string);

#define define_stage_hook(name, type, string)		 \
  static gboolean name (type *self, GError **error)	 \
  {                                                      \
    g_assert(self);                                      \
    write_resuid(string);				 \
    i_am_here();					 \
    return TRUE;                                         \
  }                                                      \

static void 
write_resuid(const gchar *string)
{
  uid_t ruid, euid, suid;
  gchar *ruids, *euids, *suids;
  GKeyFile *keyfile;
  gchar *data;
  gsize length;
  GError *error= NULL;
  
  g_assert(string); g_assert(tempdir);
  
  keyfile= g_key_file_new(); g_assert(keyfile);
  getresuid(&ruid, &euid, &suid);
  ruids= g_strdup_printf("%d", ruid);
  euids= g_strdup_printf("%d", euid);
  suids= g_strdup_printf("%d", suid);

  g_key_file_set_value(keyfile,
		       string,
		       "ruid",
		       ruids);
  g_key_file_set_value(keyfile,
		       string,
		       "euid",
		       euids);
  g_key_file_set_value(keyfile,
		       string,
		       "suid",
		       suids);

  g_free(ruids); g_free(euids); g_free(suids);

  do {
    gchar *filename;
    
    filename= g_strdup_printf("%s/%s", tempdir, string);
    g_assert(filename);
    data= g_key_file_to_data(keyfile, &length, &error);
    if (NULL == data) break;
    if (!g_file_set_contents(filename, data, length, &error)) break;
    g_free(filename); g_free(data);
  } while(0);
  if (error) {
    g_error("error: %s", error->message);
  }    
}

define_stage_hook(process_init, SandboxerProcess, "PROCESS_INSTANCE_INIT");
define_stage_hook(process_pre_fork, SandboxerProcess, "PROCESS_PRE_FORK");
define_stage_hook(process_pre_unshare, SandboxerProcess, "PROCESS_PRE_UNSHARE");
define_stage_hook(process_pre_pivot_root, SandboxerProcess, "PROCESS_PRE_PIVOT_ROOT");
define_stage_hook(process_pre_umount, SandboxerProcess, "PROCESS_PRE_UMOUNT");
define_stage_hook(process_pre_setuid, SandboxerProcess, "PROCESS_PRE_SETUID");
define_stage_hook(process_pre_exec, SandboxerProcess, "PROCESS_PRE_EXEC");
define_stage_hook(process_instance_finalize, SandboxerProcess, "PROCESS_INSTANCE_FINALIZE");

define_stage_hook(sandbox_init, SandboxerSandbox, "SANDBOX_INSTANCE_INIT");
define_stage_hook(sandbox_pre_fork, SandboxerSandbox, "SANDBOX_PRE_FORK");
define_stage_hook(sandbox_pre_unshare, SandboxerSandbox, "SANDBOX_PRE_UNSHARE");
define_stage_hook(sandbox_pre_pivot_root, SandboxerSandbox, "SANDBOX_PRE_PIVOT_ROOT");
define_stage_hook(sandbox_pre_umount, SandboxerSandbox, "SANDBOX_PRE_UMOUNT");

static void
close_func(SandboxerPlugin *plugin);

const gchar* g_module_check_init(GModule *module);

const SandboxerPluginDescriptor test_descriptor = {
  .plugin_name= "test-plugin",
  .major_version= PLUGIN_MAJOR_VERSION,
  .minor_version= PLUGIN_MINOR_VERSION,
  .process_class_init_hook= NULL, //FIXME
  .process_hooks= { 
    process_init,                 /* INSTANCE_INIT     */ 
    process_pre_fork,             /* PRE_FORK          */
    process_pre_unshare,          /* PRE_UNSHARE       */
    process_pre_pivot_root,       /* PRE_PIVOT_ROOT    */
    process_pre_umount,           /* PRE_UMOUNT        */
    process_pre_setuid,           /* PRE_SETUID        */
    process_pre_exec,             /* PRE_EXEC          */
    process_instance_finalize,    /* INSTANCE_FINALIZE */
  },
  .sandbox_class_init_hook= NULL, //FIXME
  .sandbox_hooks= {
    sandbox_init,                 /* INSTANCE_INIT    */
    sandbox_pre_fork,             /* PRE_FORK          */
    sandbox_pre_unshare,          /* PRE_UNSHARE       */
    sandbox_pre_pivot_root,       /* PRE_PIVOT_ROOT    */
    sandbox_pre_umount,           /* PRE_UMOUNT        */
  },
  .close_hook= close_func,
  .padding= PADDING_INIT
};



static void
close_func(SandboxerPlugin *plugin)
{
  g_debug("close func hi!");
}

const gchar* g_module_check_init(GModule *module)
{
  g_debug("test module in da house!");
  g_assert(module);
  g_debug("tempdir: %s", tempdir);
  if (sandboxer_register_plugin(&test_descriptor, module)) {
    g_debug("yes"); 
  } else {
    g_debug("no");
  }

  return NULL;
}

