/*
 * application-sandboxer: an application isolation mechanism
 * Copyright (C) 2009, Intel Corporation.
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms and conditions of the GNU Lesser General Public License,
 * version 2.1, as published by the Free Software Foundation.
 *
 * This program is distributed in the hope it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
 * more details.
 *
 * You should have received a copy of the GNU Lesser General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51 Franklin St 
 * - Fifth Floor, Boston, MA 02110-1301 USA
 *
 */


#include <config.h>
#include <test/check_helper.h>
#include <test/check_process.h>
#include <test/check_sandboxer_service.h>
#include <test/check_client_server.h>
#include <test/check_config_store.h>
#include <test/check_sandbox.h>
#include <test/check_plugin.h>
#include <test/check_env_resolver.h>

#include <daemon/sandboxer-helper.h>
#include <daemon/config_store.h>
#include <test/config_file_helper.h>

#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <pwd.h>
#include <errno.h>
#include <glib.h>
#include <glib/gthread.h>
#include <check.h>

#define CONFIG_SUITE_ENV  "CONFIG_SUITE"
#define PROCESS_SUITE_ENV "PROCESS_SUITE"
#define SANDBOX_SUITE_ENV "SANDBOX_SUITE"
#define SANDBOXER_SUITE_ENV "SANDBOXER_SUITE"
#define PLUGIN_SUITE_ENV "PLUGIN_SUIT"
#define ENV_RESOLVER_SUITE_ENV "ENV_RESOLVER_SUITE"
#define CLIENT_SERVER_SUITE_ENV "CLIENT_SERVER_SUITE"

uid_t sandboxer_uid;
gid_t sandboxer_gid;
uid_t tester_uid;
gid_t tester_gid;
GPtrArray *plugins;

int main(void)
{
  int number_failed;
  Suite *s;
  SRunner *sr;
  uid_t helper_uid;
  gid_t helper_gid;
  gchar *plugins_path;
  enum suites {
    CONFIG_SUITE = 1,
    PROCESS_SUITE= 1<<1,
    SANDBOX_SUITE= 1<<2,
    SANDBOXER_SUITE= 1<<3,
    PLUGIN_SUITE= 1<<4,
    ENV_RESOLVER_SUITE= 1<<5,
    CLIENT_SERVER_SUITE= 1<<6
  }enabled_suites;
  
  enabled_suites= 0;
  if (g_getenv(CONFIG_SUITE_ENV)) enabled_suites|= CONFIG_SUITE; 
  if (g_getenv(PROCESS_SUITE_ENV)) enabled_suites|= PROCESS_SUITE; 
  if (g_getenv(SANDBOX_SUITE_ENV)) enabled_suites|= SANDBOX_SUITE; 
  if (g_getenv(PLUGIN_SUITE_ENV)) enabled_suites|= PLUGIN_SUITE; 
  if (g_getenv(ENV_RESOLVER_SUITE_ENV)) enabled_suites|= ENV_RESOLVER_SUITE; 
  if (g_getenv(CLIENT_SERVER_SUITE_ENV)) enabled_suites|= CLIENT_SERVER_SUITE; 
  if (!enabled_suites) {
    enabled_suites= CONFIG_SUITE | PROCESS_SUITE | SANDBOX_SUITE |
      SANDBOXER_SUITE | PLUGIN_SUITE | ENV_RESOLVER_SUITE | CLIENT_SERVER_SUITE;
  }
  
  putenv("G_DEBUG=fatal_warnings");
  
  g_type_init();

  if (!g_thread_supported ())
    g_thread_init (NULL);
  dbus_g_thread_init ();
  
  if (!is_root()) {
    g_error("Please run as root!");
  }

  get_uidgid("tester", &tester_uid, &tester_gid);
  get_uidgid("sandboxer", &sandboxer_uid, &sandboxer_gid);

  
  plugins_path= g_strdup_printf("%s/../plugins/.libs/", g_get_current_dir());

  plugins= g_ptr_array_new();

  g_setenv("CK_DEFAULT_TIMEOUT", "60", TRUE);

  s= helper_suite ();
  sr= srunner_create (s);
  if (enabled_suites & CONFIG_SUITE) {
    g_print("Enabling config suite\n");
    srunner_add_suite(sr, config_suite());
  }
  if (enabled_suites & PROCESS_SUITE) {
    g_print("Enabling process suite\n");
    srunner_add_suite(sr, process_suite());
  }
  if (enabled_suites & SANDBOX_SUITE) {
    g_print("Enabling sandbox suite\n");
    srunner_add_suite(sr, sandbox_suite());
  }
  if (enabled_suites & SANDBOXER_SUITE) {
    g_print("Enabling sandboxer suite\n");
    srunner_add_suite(sr, sandboxer_suite());
  }
  if (enabled_suites & PLUGIN_SUITE) {
    g_print("Enabling plugin suite\n");
    srunner_add_suite(sr, plugin_suite()); 
  }
  if (enabled_suites & ENV_RESOLVER_SUITE) {
    g_print("Enabling env resolver suite\n");
    srunner_add_suite(sr, env_resolver_suite()); 
  }
  if (enabled_suites & CLIENT_SERVER_SUITE) {
    g_print("Enabling client server suite\n");
    srunner_add_suite(sr, client_server_suite());
  }
  srunner_run_all (sr, CK_NORMAL);
  number_failed= srunner_ntests_failed (sr);
  srunner_free (sr);
  
  return (number_failed == 0) ? EXIT_SUCCESS : EXIT_FAILURE;
}
