import subprocess
import unittest
import time
import xmlrpclib
import signal
import os

from helpers import Command
from helpers import SandboxerTestCase

# need sudo in jail
#  option 1) require group and do it by hand
#  option 2) modify sudoers programmatically
#            a) bizzaro visudo hack
#               make splice in and splice out scripts
#               invoke them as "editors"
#            b) just do it
#
#


# 1. install sandbox package
# 2. create a new jail
# 3. verify items listed under DirsInJail in sandboxerd.conf exist
# 4. verify that no other items are present in jail
# 5. verify that old_root is unmounted
# 6. create 2 more jails and run steps 3-5 (?!)
# 7. verify that mounts and unmounts from the host do not propogate
# 8. verify that mounts and unmounts done from the three jails do not propagate
# 9. verify that files wrriten are within jail confines
class TestJailSecurity(SandboxerTestCase):
    """Test Case 4 as in the Test Case document.
    
    Step 1 is an assumed to be completed before running.  
    Additionally, the package listdirs is expected to be installed.

    """

    def testDirsInJail(self):        
        """Test Case 4.3 and 4.4"""
        DIRS_IN_JAIL= set(['bin', 'usr', 'lib', 'opt', 'tmp', 'var', 
                           'etc', 'dev', 'proc', 'sys', 'old_root']) # as given in sandboxerd.conf plus old_root


        output= {}
        program= Command('/usr/bin/sandboxer /opt/list_dirs/bin/list_dirs', output)
        program.start()
        time.sleep(1) # settling time
        s = xmlrpclib.Server('http://localhost:54321')
        dirs_in_jail= set(s.look_around())
        program.kill(signal.SIGTERM)
        self.failUnlessEqual(DIRS_IN_JAIL, dirs_in_jail, 
                             "symetric difference in dirs: %s" % repr(DIRS_IN_JAIL.symmetric_difference(dirs_in_jail)))

    def testOldRoot(self):
        """Test Case 4.5: verify that old_root is unmounted"""
        output= {}
        program= Command('/usr/bin/sandboxer /opt/list_dirs/bin/list_dirs', output)
        program.start()
        time.sleep(1) # settling time
        s = xmlrpclib.Server('http://localhost:54321')
        self.failUnlessEqual([0, '', ''], s.sudo_command('ls /old_root'))
        program.kill(signal.SIGTERM)
        
    def testMountsDoNotPropogateToSandbox(self):
        """Test Case 4.7: verify that mounts and umounts from the host do not propogate"""

        MOUNT_POINT= "/tmp/mount_point"
        FILE= MOUNT_POINT + "/file"

        if not os.path.exists(MOUNT_POINT):
            os.mkdir(MOUNT_POINT)
        output= {}
        program= Command('/usr/bin/sandboxer /opt/list_dirs/bin/list_dirs', output)
        program.start()
        time.sleep(1) # settling time
        subprocess.check_call(['mount', '-t', 'tmpfs', '-osize=1M', 'tmpfs', MOUNT_POINT])
        subprocess.check_call(['touch', FILE])
        s = xmlrpclib.Server('http://localhost:54321')
        exists= s.check_exists(FILE)
        subprocess.check_call(['umount', MOUNT_POINT])
        self.failIf(exists)

    def testMountsDoNotProporateFromSandbox(self):
        """Test Case 4.8: verify that mounts and unmounts done from the jails do not propagate"""

        MOUNT_POINT= "/tmp/mount_point"
        FILE= MOUNT_POINT + "/file"

        if not os.path.exists(MOUNT_POINT):
            os.mkdir(MOUNT_POINT)
        output= {}
        program= Command('/usr/bin/sandboxer /opt/list_dirs/bin/list_dirs', output)
        program.start()
        time.sleep(1) # settling time
        s = xmlrpclib.Server('http://localhost:54321')
        self.failUnlessEqual([0, '', ''], s.sudo_command("mount -t tmpfs -osize=1M tmpfs " + MOUNT_POINT))
        self.failUnlessEqual([0, '', ''], s.sudo_command("touch " + FILE))
        self.failIf(os.path.exists(FILE))

    def tearDown(self, result=None):
        SandboxerTestCase.tearDown(self, result)
        subprocess.call(['killall', 'sandboxer']) #this doesn't work at all
         
if __name__ == '__main__':
    unittest.main()
