/*
 * application-sandboxer: an application isolation mechanism
 * Copyright (C) 2009, Intel Corporation.
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms and conditions of the GNU Lesser General Public License,
 * version 2.1, as published by the Free Software Foundation.
 *
 * This program is distributed in the hope it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
 * more details.
 *
 * You should have received a copy of the GNU Lesser General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51 Franklin St 
 * - Fifth Floor, Boston, MA 02110-1301 USA
 *
 */


#include <sys/types.h>
#include <unistd.h>
#include <signal.h>
#include <stdio.h>
#include <stdlib.h>
#include <assert.h>
#include <errno.h>

const char* fname= "/tmp/check_sandboxer_service.pid";

void cleanup(int foo)
{
  unlink(fname);  
  exit(EXIT_SUCCESS);
}

void nuke_old_selves()
{
  FILE *pidfile;
  pid_t oldpid= 0;

  pidfile= fopen(fname, "r");
  if (NULL == pidfile) {
    if (ENOENT == errno) {
      return;
    } else {
      exit(EXIT_FAILURE);
    }
  }
  fscanf(pidfile, "%d", &oldpid);
  printf("oldpid is %d\n", oldpid);
  fclose(pidfile);
  kill(oldpid, SIGKILL);  
}

int main(void)
{
  pid_t mypid;
  FILE* pidfile;

  nuke_old_selves();

  pidfile= fopen(fname, "w");
  assert(pidfile);
  mypid= getpid();
  fprintf(pidfile, "%d\n", mypid);
  fclose(pidfile);
  sync();

  assert(SIG_ERR != signal(SIGTERM, cleanup));
  assert(SIG_ERR != signal(SIGINT, cleanup));

  /* the big sleep */
  while(1) {
    sleep(1000);
  }
  
  /* control never reaches here */
  return 0;
}
