/*
 * application-sandboxer: an application isolation mechanism
 * Copyright (C) 2009, Intel Corporation.
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms and conditions of the GNU Lesser General Public License,
 * version 2.1, as published by the Free Software Foundation.
 *
 * This program is distributed in the hope it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
 * more details.
 *
 * You should have received a copy of the GNU Lesser General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51 Franklin St 
 * - Fifth Floor, Boston, MA 02110-1301 USA
 *
 */


#include <config.h>
#include <stdlib.h>
#include <glib.h>
#include <sandboxer-service-glue.h>


int main(int argc, char **argv)
{
  GError *error = NULL;
  DBusGConnection *connection;
  DBusGProxy *sandboxer_proxy;
  enum { ADD, REMOVE } mode;

  g_type_init();
  if (!g_thread_supported ())
    g_thread_init (NULL);
  dbus_g_thread_init ();
  
  if (g_str_has_suffix(argv[0], "add_package")) {
    mode= ADD;
  } else if (g_str_has_suffix(argv[0], "remove_package")) {
    mode= REMOVE;
  } else {
    g_printerr("Unknown mode\n");
    exit(EXIT_FAILURE);
  }
  
  if (argc < 2) {
    if (ADD == mode) 
      g_printerr("Usage: %s PACKAGE_FILE\n\n", argv[0]);
    if (REMOVE == mode)
      g_printerr("Usage: %s PACKAGE_NAME\n\n", argv[0]);
    exit(EXIT_FAILURE);
  }
  connection = dbus_g_bus_get (DBUS_BUS_SYSTEM, &error);
  if (!connection) {
    g_printerr("could not get connection to DBUS Session: %s\n\n", error->message);
    exit(EXIT_FAILURE);
  }
  sandboxer_proxy = dbus_g_proxy_new_for_name (connection,
					       "com.intel.sandboxer.SandboxService",
					       "/com/intel/sandboxer/SandboxService",
					       "com.intel.sandboxer.Query");
  
  if (!sandboxer_proxy) {
    g_error("Could not get sandboxer_proxy for "
	    "com.intel.sandboxer.Query");
  }
  if (ADD == mode) {
    gchar *filename; 
    
    filename= argv[1];
    g_assert(filename);
    
    if (!com_intel_sandboxer_Query_add_package_file (sandboxer_proxy,
						     filename,
						     &error)) {
      g_printerr("add_package: %s\n\n", error->message);
      exit(EXIT_FAILURE);
    }
  } else {
    gchar *package_name;
    
    package_name= argv[1];
    g_assert(package_name);
    if (!com_intel_sandboxer_Query_remove_package(sandboxer_proxy,
						  package_name,
						  &error)) {
      g_printerr("remove_package: %s\n\n", error->message);
      exit(EXIT_FAILURE);
    }
  }
  exit(EXIT_SUCCESS);
}
  
